// 005 u_c_variable_record_file
// 10 dec 2004

// -- (C) Felix John COLIBRI 2004
// -- documentation: http://www.felix-colibri.com

// -- double file:
// --  fixed with (position)
// --  variable with (id, size, data)

(*$r+*)

unit u_c_variable_record_file;
  interface
    uses u_c_file_of,
        u_c_basic_file;

    type c_variable_record_file= class(c_file_of)
                                   public
                                     m_c_variable_file: c_basic_file;

                                     m_variable_position_record: Integer;

                                     m_first_free_position: Integer;

                                     // -- max record
                                     // m_variable_record_max: Integer;

                                     Constructor create_variable_record_file(p_name, p_file_name: String); Virtual;

                                     function f_force_create: Boolean; Override;
                                     function f_open: Boolean; Override;

                                     procedure append_record(p_pt: Pointer; p_size: Integer);
                                     procedure allocate_and_read_record(p_position: Integer;
                                         var pv_size: Integer; var pv_pt: Pointer);

                                     procedure display_fixed_file;

                                     procedure read_record_max(p_position: Integer;
                                         p_size_max: Integer; var pv_size: Integer; p_pt: Pointer);

                                     function f_compute_record_max: Integer;
                                     function f_variable_byte_count: Integer;

                                     procedure close_file; Override;

                                     Destructor Destroy; Override;
                                 end; // c_variable_record_file


  implementation
    uses SysUtils, Windows
        , u_c_file_name
        , u_c_display;

    // -- c_variable_record_file

    Constructor c_variable_record_file.create_variable_record_file(p_name, p_file_name: String);
      var l_c_variable_name: c_file_name;
      begin
        Inherited create_file_of(p_name, p_file_name,
            @ m_variable_position_record, SizeOf(m_variable_position_record));

        // -- create the variable file
        with m_c_file_name do
          l_c_variable_name:= f_c_file_name('name', m_drive, m_path, m_name+ '_var', m_extension);

        m_c_variable_file:= c_basic_file.create_basic_file(p_name+ '_var', l_c_variable_name.m_complete_name);

        l_c_variable_name.Free;
        // display(m_c_variable_file.m_c_file_name.m_complete_name);
      end; // create_variable_record_file

    function c_variable_record_file.f_force_create: Boolean;
      begin
        Result:= Inherited f_force_create and m_c_variable_file.f_force_create;

        with m_c_variable_file do
        begin
          m_first_free_position:= 0;
          f_block_write(@ m_first_free_position, SizeOf(m_first_free_position));
        end;  
      end; // f_force_create

    function c_variable_record_file.f_open: Boolean;
      begin
        Result:= Inherited f_open and m_c_variable_file.f_open;
      end; // f_open

    procedure c_variable_record_file.append_record(p_pt: Pointer; p_size: Integer);
      var l_id: Integer;
      begin
        l_id:= f_seek_end_of_file;
        // display('at '+ IntToStr(l_id)+ ' write '+ IntToStr(p_size));

        // -- write the variable part
        with m_c_variable_file do
        begin
          m_variable_position_record:= f_seek_end_of_file;

          // -- write id
          f_block_write(@ l_id, 4);

          // -- write the size
          f_block_write(@ p_size, 4);

          // -- write the data
          f_block_write(p_pt, p_size);
        end; // with m_c_variable_file

        write_record;
      end; // append_record

    procedure c_variable_record_file.allocate_and_read_record(p_position: Integer;
        var pv_size: Integer; var pv_pt: Pointer);
      var l_id: Integer;
      begin
        // -- get the position of the variable record
        seek_file(p_position);
        read_record;

        with m_c_variable_file do
        begin
          // -- skip the id
          seek_file(m_variable_position_record);
          f_block_read(@ l_id, 4);

          // -- get the size
          f_block_read(@ pv_size, 4);

          // -- allocate by GetMem
          GetMem(pv_pt, pv_size);
          // display(' aha '+ IntToStr(pv_size));

          // -- get the variable record
          f_block_read(pv_pt, pv_size);
        end; // with m_c_variable_file
      end; // allocate_and_read_record

    procedure c_variable_record_file.read_record_max(p_position: Integer;
        p_size_max: Integer; var pv_size: Integer; p_pt: Pointer);
      var l_id: Integer;
      begin
        // -- get the position of the variable record
        seek_file(p_position);
        read_record;

        with m_c_variable_file do
        begin
          // -- skip the id
          seek_file(m_variable_position_record);
          f_block_read(@ l_id, 4);

          // -- get the size
          f_block_read(@ pv_size, 4);

          // -- ++ check MAX !!!

          // -- get the variable record
          f_block_read(p_pt, pv_size);
        end; // with m_c_variable_file
      end; // read_record_max

    procedure c_variable_record_file.display_fixed_file;
        // -- debug: only display fixed file
      begin
        Inherited f_open;

        while not f_eof do
        begin
          inherited read_record;
          display(IntToStr(m_variable_position_record));
        end;

        Inherited close_file;
      end; // display_fixed_file

    function c_variable_record_file.f_compute_record_max: Integer;
        // -- compute the maximum size in the fixed file
      var l_previous_position, l_size: Integer;
      begin
        Result:= 0;

        seek_file(0);
        l_previous_position:= 0;

        while not f_eof do
        begin
          inherited read_record;

          if l_previous_position<> 0
            then begin
                l_size:= m_variable_position_record- l_previous_position;
                if l_size> Result
                  then Result:= l_size;
              end;
          l_previous_position:= m_variable_position_record;
        end; // while not f_eof

        // ++ compute last record

        seek_file(0);
      end; // f_compute_record_max

    function c_variable_record_file.f_variable_byte_count: Integer;
      begin
        Result:= m_c_variable_file.f_byte_count;
      end; // f_variable_byte_count

    procedure c_variable_record_file.close_file;
      begin
        Inherited;
        m_c_variable_file.close_file;
      end; // close_file

    Destructor c_variable_record_file.Destroy;
      begin
        Inherited;
      end; // Destroy

    begin // u_c_variable_record_file
    end. // u_c_variable_record_file
file
