// 003 u_c_sniffer
// 10 dec 2004

// -- (C) Felix John COLIBRI 2004
// -- documentation: http://www.felix-colibri.com

(*$r+*)

unit u_c_sniffer;
  interface
    uses Windows, Classes, Sysutils
        , u_c_basic_object
        , u_packet_capture
        ;

    type c_sniffer_thread= class; // forward

         c_sniffer= Class(c_basic_object)
                      // -- pointer to the capture driver
                      m_pt_capture_info: t_pt_capture_info;

                      // -- the adapters found on the system
                      m_c_adapter_name_list: TstringList;
                      m_current_adapter_index: Integer;

                      // -- the listening thread
                      _m_c_sniffer_thread: c_sniffer_thread;

                      m_is_snooping: Boolean;

                      // -- the event called whenever a packet is received
                      m_on_sniffer_received_packet: Procedure (Data: pointer; recvbytes: Word) of Object ;

                      Constructor create_sniffer(p_name: String);
                      Function f_get_adapters(Var pv_error_string: string): boolean;
                      Function f_activate(var pv_error_string: string): boolean;
                      Function f_deactivate(var pv_error_string: string): boolean;
                      procedure sniffer_thread_did_terminate(Sender: tobject);
                      Destructor Destroy; override;
                    end; // c_sniffer

         // -- Thread that listens to selected adapter
         c_sniffer_thread= Class(Tthread)
                             private
                               m_c_sniffer: c_sniffer;
                             public
                               Constructor Create(p_c_sniffer: c_sniffer);
                               Destructor Destroy; override;
                               Procedure Execute; override;
                           end; // c_sniffer_thread

  implementation
    uses u_c_display;

    // -- c_sniffer

    constructor c_sniffer.create_sniffer(p_name: String);
      begin
        inherited create_basic_object(p_name);

        m_c_adapter_name_list:= Tstringlist.Create;
      end; // create_sniffer

    function c_sniffer.f_get_adapters(var pv_error_string: string): boolean;
      begin
        Result:= false;

        if m_c_adapter_name_list= nil
          then
            begin
              pv_error_string:= 'Memory for Adapterlist not allocated';
              Exit;
            end;

        m_c_adapter_name_list.commatext:= pcap_GetAdapternames(',', pv_error_string);

        if m_c_adapter_name_list.CommaText= ''
          then Exit;
        Result:= true;
      end; // f_get_adapters

    function c_sniffer.f_activate(var pv_error_string: string): boolean;
        // -- f_activate sniff, start thread
      begin
        Result:= false;

        // -- Check if Snooping is active
        if m_is_snooping or (m_pt_capture_info<> nil)
          then
            begin
              pv_error_string:= 'Snooping already activated';
              Exit;
            end;

        // -- Open Driver and NetAdapter
        // --  (..., promiscuous, timeout, )
        m_pt_capture_info:= pcap_open_live(Pchar(m_c_adapter_name_list[m_current_adapter_index]),
            k_default_snapshot_length, True, 100, pv_error_string);
        if m_pt_capture_info= nil
          then begin
              pv_error_string:= 'pcap_handle NIL';
              Exit;
            end;

        if not Assigned(m_on_sniffer_received_packet)
          then
            begin
              pv_error_string:= 'No Packet Read Callback function assigned';
              Exit;
            end;

        // -- start the snooping thread
        _m_c_sniffer_thread:= c_sniffer_thread.create(self);
        _m_c_sniffer_thread.OnTerminate:= sniffer_thread_did_terminate;
        _m_c_sniffer_thread.FreeOnTerminate:= False;

        // -- now start the thread
        _m_c_sniffer_thread.Resume;
        m_is_snooping:= True;

        Result:= true;
      end; // f_activate

    procedure c_sniffer.sniffer_thread_did_terminate(Sender: tobject);
      begin
        m_is_snooping:= false;
      end; // sniffer_thread_did_terminate

    function c_sniffer.f_deactivate(var pv_error_string: string): boolean;
      begin
        display('> f_deactivate');
        Result:= false;

        if not m_is_snooping
          then
            begin
              pv_error_string:= 'Snooping not active';
              display('< f_deactivate. not_snooping. Exit ');
              Exit;
            end;

        if _m_c_sniffer_thread= nil
          then
            begin
              pv_error_string:= 'No thread to stop';
              display('< f_deactivate. no_thread. Exit ');
              Exit;
            end;

        // -- Stop Snooping Thread
        _m_c_sniffer_thread.Terminate;
        _m_c_sniffer_thread.WaitFor;
        _m_c_sniffer_thread.Free;
        _m_c_sniffer_thread:= nil;

        // -- Release Driver Handle
        Pcap_Close(m_pt_capture_info);

        Result:= true;
        display('< f_deactivate');
      end; // f_deactivate

    destructor c_sniffer.Destroy;
      Var l_error_string: string;
      begin
        f_deactivate(l_error_string);

        m_c_adapter_name_list.free;
        m_c_adapter_name_list:= nil;

        inherited;
      end; // Destroy

    // -- the call back

    procedure capture_call_back(p_c_sniffer: pointer; const p_pcap_header: Ppcap_pkthdr;
        const p_pt_data: pchar);
        // -- Snoop Driver Callback function cannot be procedure of object
        // --  => this procedure will call the event
      begin
        c_sniffer(p_c_sniffer).m_on_sniffer_received_packet(p_pt_data, p_pcap_header.len);
      end; // capture_call_back

    // -- c_sniffer_thread

    constructor c_sniffer_thread.Create(p_c_sniffer: c_sniffer);
      begin
        m_c_sniffer:= p_c_sniffer;

        // -- set the timeout
        pcap_set_read_timeout(m_c_sniffer.m_pt_capture_info, 100);

        inherited Create(True);
      end; // Create

    procedure c_sniffer_thread.Execute;
      begin
        if m_c_sniffer= nil
          then Exit;

        // -- whenever a packet is detected, call our event
        While Not Terminated do
          Pcap_Read(m_c_sniffer.m_pt_capture_info, 0, capture_call_back, Pointer(m_c_sniffer));
      end; // Execute

    destructor c_sniffer_thread.Destroy;
      begin
        inherited;
      end; // Destroy

    end.
end.
