// 004 u_sniffer_utils
// 04 feb 2005

// -- (C) Felix John COLIBRI 2004
// -- documentation: http://www.felix-colibri.com

// -- some TCP string routines

(*$r+*)

unit u_sniffer_utils;
  interface

    const // well-known ports
          k_tcp_dns= 53;
          k_udp_netbios_name_service= 137;
          k_udp_netbios_datagram= 138;

    function f_swap_word(p_x: pChar): Word;
    function f_swap_dword(p_x: PChar): Integer;

    type t_MAC_address= array[0..5] of BYTE;
         t_IP_address= array[0..3] of BYTE;

    function f_MAC_to_string(p_MAC_address: t_MAC_address): string;
    function f_IP_to_string(IPAddr: t_IP_address): string;
    function f_IP_to_string_hex(p_IP_address: t_IP_address): string;
    procedure initialize_our_mac_address(p_0, p_1, p_2, p_3, p_4, p_5: Byte);

    function f_normalize_url(p_url: String): String;

    const k_reception= 'i';
          k_emission= 'o';
          k_other= '?';

    var g_MAC_address: t_MAC_address;
        g_ppp_receive_mac, g_ppp_send_mac: t_MAC_address;

    procedure display_string_indented(p_indentation: Integer; p_string: String);

    procedure display_5_columns(p_id, p_hex, p_protocol, p_sub_protocol, p_detail: String);
    procedure display_4_columns(p_id, p_protocol, p_sub_protocol, p_detail: String);

    function f_port(p_port: Word): String;
    var g_show_unknown_ports: Boolean= False;

    type t_packet_type= (e_ppp_in, e_ppp_out, e_ppp_pass,
            e_arp_in, e_arp_out, e_arp_pass,
            e_ip_icmp_in, e_ip_icmp_out, e_ip_icmp_pass,
            e_ip_udp_in, e_ip_udp_out, e_ip_udp_pass,
            e_ip_tcp_in, e_ip_tcp_out, e_ip_tcp_pass);
         t_packet_type_set= Set of t_packet_type;

    function f_display_packet_type(p_packet_type: t_packet_type): String;
    function f_display_packet_type_set(p_packet_type_set: t_packet_type_set): String;

  implementation
    Uses Classes, SysUtils, Math, TypInfo, Dialogs
      , u_types_constants, u_strings, u_c_display;

    function f_add_to_the_line(p_line: Integer; p_c_stringlist: tStringList; p_length: Integer): String;
      begin
        if p_line< p_c_stringlist.Count
          then Result:= p_c_stringlist[p_line]
          else Result:= '';

        if p_length<> -1
          then Result:= Format('%-'+ IntToStr(p_length)+ 's', [Result]);
      end; // f_add_to_the_line

    procedure display_5_columns(p_id, p_hex, p_protocol, p_sub_protocol, p_detail: String);
      // -- receives strings with Returns. Format it in 5 columns
      var l_c_id, l_c_hex, l_c_protocol, l_c_sub_protocol, l_c_detail: tStringList;
          l_line: Integer;
          l_line_max: Integer;
      begin
        l_c_id:= tStringList.Create; l_c_hex:= tStringList.Create;
        l_c_protocol:= tStringList.Create; l_c_sub_protocol:= tStringList.Create; l_c_detail:= tStringList.Create;

        l_c_id.Text:= p_id; l_c_hex.Text:= p_hex; l_c_protocol.Text:= p_protocol;
        l_c_sub_protocol.Text:= p_sub_protocol; l_c_detail.Text:= p_detail;

        // -- count the max, but NOT the hex ?
        l_line_max:= Max(l_c_id.Count, Max(l_c_hex.Count, Max(l_c_protocol.Count, Max(l_c_sub_protocol.Count, l_c_detail.Count))))- 1;
        for l_line:= 0 to l_line_max do
          display(f_add_to_the_line(l_line, l_c_id, 4+ 3)
              + f_add_to_the_line(l_line, l_c_hex, 8* 3+ 1)
              + f_add_to_the_line(l_line, l_c_protocol, 5)
              + f_add_to_the_line(l_line, l_c_sub_protocol, 5)
              + f_add_to_the_line(l_line, l_c_detail, -1));

        l_c_id.Free; l_c_hex.Free; l_c_protocol.Free; l_c_sub_protocol.Free; l_c_detail.Free;
      end; // display_5_columns

    procedure display_4_columns(p_id, p_protocol, p_sub_protocol, p_detail: String);
      var l_c_id, l_c_protocol, l_c_sub_protocol, l_c_detail: tStringList;
          l_line: Integer;
          l_line_max: Integer;
      begin
        l_c_id:= tStringList.Create;
        l_c_protocol:= tStringList.Create; l_c_sub_protocol:= tStringList.Create; l_c_detail:= tStringList.Create;

        l_c_id.Text:= p_id; l_c_protocol.Text:= p_protocol;
        l_c_sub_protocol.Text:= p_sub_protocol; l_c_detail.Text:= p_detail;

        l_line_max:= Max(l_c_id.Count, Max(l_c_protocol.Count, Max(l_c_sub_protocol.Count, l_c_detail.Count)))- 1;
        for l_line:= 0 to l_line_max do
          display(f_add_to_the_line(l_line, l_c_id, 4+ 3)
              + f_add_to_the_line(l_line, l_c_protocol, 5)
              + f_add_to_the_line(l_line, l_c_sub_protocol, 5)
              + f_add_to_the_line(l_line, l_c_detail, -1));

        l_c_id.Free; l_c_protocol.Free; l_c_sub_protocol.Free; l_c_detail.Free;
      end; // display_4_columns

    (*$r-*)

    function f_swap_word(p_x: pChar): Word;
      begin
        Result:= (Word(p_x^) shl 8) or (Word((p_x+ 1)^));
      end; // f_swap_word

    function f_swap_dword(p_x: PChar): Integer;
      begin
        Result:= (Integer(p_x^) shl 24) or
          (Integer((p_x+ 1)^) shl 16) or
          (Integer((p_x+ 2)^) shl 8) or
          (Integer((p_x+ 3)^));
      end; // f_swap_dword

    (*$r+*)

    function f_MAC_to_string(p_MAC_address: t_MAC_address): string;
      var l_the_direction: String[5];
          i : integer;
      begin
        l_the_direction:= '     ';
        move(p_MAC_address, l_the_direction[1], 5);
        if l_the_direction= ' SEND'
          then Result:= '_SEND_'+ f_spaces(11)
          else
            if l_the_direction= ' RECV'
              then Result:= '_RECV_'+ f_spaces(11)
              else begin
                  l_the_direction:= '    ';
                  move(p_MAC_address, l_the_direction[1], 4);
                  // display(l_the_direction);

                  if l_the_direction= 'DEST'
                    then Result:= 'DEST__'+ f_spaces(12)
                    else begin
                        Result := '';
                        for i := low(p_MAC_address) to High(p_MAC_address) do
                          Result := Result + IntToHex(p_MAC_address[i], 2)+ '-';

                        // -- remove the last "-"
                        Delete(Result, Length(Result), 1);
                      end;
                end;
      end; // f_MAC_to_string

    function f_IP_to_string(IPAddr: t_IP_address): string;
      var i : integer;
      begin
        Result := '';
        for i := Low(IPAddr) to High(IPAddr) do
           Result := Result + IntToStr(IPAddr[i]) + '.';
        Delete(Result, Length(Result), 1);
      end; // f_IP_to_string

    function f_IP_to_string_hex(p_IP_address: t_IP_address): string;
      var l_index: integer;
      begin
        Result := '';
        for l_index:= Low(p_IP_address) to High(p_IP_address) do
           Result := Result + IntToHex(p_IP_address[l_index], 2) + '.';
        Delete(Result, Length(Result), 1);
      end; // f_IP_to_string_hex

    procedure display_string_indented(p_indentation: Integer; p_string: String);
      var l_line: Integer;
          l_text: String;
          l_c_string_list: tStringList;
      begin
        l_text:= '';
        l_c_string_list:= tStringList.Create;
        l_c_string_list.Text:= p_string;

        Inc(g_c_display.m_indentation, p_indentation);
        for l_line:= 0 to l_c_string_list.Count- 1 do
          display('|'+ l_c_string_list[l_line]);
        Dec(g_c_display.m_indentation, p_indentation);

        l_c_string_list.Free;
      end; // display_string_indented

    function f_normalize_url(p_url: String): String;
        // -- "12.1.456.18" -> "_12.__1.456._18"
      var l_index: Integer; // , l_segment_index, l_result_index, l_fill: Integer;
          l_segment: String;
      begin
        Result:= '';

        l_segment:= '';

        for l_index:= 1 to Length(p_url) do
        begin
          if p_url[l_index]= '.'
            then begin
                l_segment:= f_spaces(3- Length(l_segment))+ l_segment;
                Result:= Result+ l_segment+ '.';
                l_segment:= '';
              end
            else l_segment:= l_segment+ p_url[l_index];
        end; // for

        // -- last part
        l_segment:= f_spaces(3- Length(l_segment))+ l_segment;
        Result:= Result+ l_segment;
      end; // f_normalize_url

    var s_c_unknown_ports: tStringList= Nil;

    function f_port(p_port: Word): String;
      begin
        if s_c_unknown_ports= Nil
          then s_c_unknown_ports:= tStringList.Create;

        Case p_port of
          53 : Result:= 'TCP_DNS';
          80 : Result:= 'HTTP';
          110 : Result:= 'TCP_pop3';
          119 : Result:= 'TCP_nntp';
          137 : Result:= 'NetBios_name_service';
          138 : Result:= 'NetBios_datagram';
          139 : Result:= 'NetBios_session_service';
          else
            Result:= IntToStr(p_port);

            if s_c_unknown_ports.IndexOf(Result)< 0
              then begin
                  s_c_unknown_ports.Add(Result);
                  if g_show_unknown_ports
                    then stop('unknown_port '+ Result);
                end;
        end; // case
      end; // f_port

    procedure initialize_standard_mac;
      begin
        // -- PPP mac adress (run WinIPCfg)

        // -- MAC address pc-1400
        // -- XP: start/accessoires | tools etc
        g_MAC_address[0]:= $00;
        g_MAC_address[1]:= $10;
        g_MAC_address[2]:= $18;
        g_MAC_address[3]:= $A5;
        g_MAC_address[4]:= $4D;
        g_MAC_address[5]:= $C2;

        g_ppp_receive_mac[0]:= $20;
        g_ppp_receive_mac[1]:= Ord('R');
        g_ppp_receive_mac[2]:= Ord('E');
        g_ppp_receive_mac[3]:= Ord('C');
        g_ppp_receive_mac[4]:= Ord('V');
        g_ppp_receive_mac[5]:= $00;

        g_ppp_send_mac[0]:= $20;
        g_ppp_send_mac[1]:= Ord('S');
        g_ppp_send_mac[2]:= Ord('E');
        g_ppp_send_mac[3]:= Ord('N');
        g_ppp_send_mac[4]:= Ord('D');
        g_ppp_send_mac[5]:= $00;
      end; // initialize_standard_mac

    procedure initialize_our_mac_address(p_0, p_1, p_2, p_3, p_4, p_5: Byte);
      begin
        g_MAC_address[0]:= p_0;
        g_MAC_address[1]:= p_1;
        g_MAC_address[2]:= p_2;
        g_MAC_address[3]:= p_3;
        g_MAC_address[4]:= p_4;
        g_MAC_address[5]:= p_5;
      end; // initialize_our_mac_address

    function f_display_packet_type(p_packet_type: t_packet_type): String;
      begin
        Result:= GetEnumName(TypeInfo(t_packet_type), Integer(p_packet_type));
        if p_packet_type in [e_ppp_in, e_ppp_out, e_ppp_pass, e_arp_in, e_arp_out, e_arp_pass]
          then Result:= Copy(Result, 3, 5) else
        if p_packet_type in [e_ip_icmp_in, e_ip_icmp_out, e_ip_icmp_pass]
          then Result:= Copy(Result, 6, 6)
          else Result:= Copy(Result, 6, 5)
      end; // f_display_packet_type

    function f_display_packet_type_set(p_packet_type_set: t_packet_type_set): String;
      var l_packet_type: t_packet_type;
      begin
        Result:= '';
        for l_packet_type:= e_ppp_in to e_ip_tcp_pass do
          if l_packet_type IN p_packet_type_set
            then Result:= Result+ f_display_packet_type(l_packet_type)+ ', ';
        if Result<> ''
          then Delete(Result, Length(Result)- 1, 2);
      end; // f_display_packet_type_set

    begin
      initialize_standard_mac
    end.

end.
