// 002 u_packet_capture
// 23 nov 2004

//.no_cr

// -- calls the packet32 wrapper

// -- written by Lars Peter Christiansen

(*$r+*)

unit u_packet_capture;
  interface
    uses Windows
      , SysUtils
      , u_ndis
      , u_packet_format // italy packet filter format
      , u_packet_32; // the driver import unit

    const PCAP_ERRBUF_SIZE= 256; // String size of error descriptions

          // -- data link constants
          DLT_NULL= 0; // no link-layer encapsulation 
          DLT_EN10MB= 1; // Ethernet (10Mb) 
          DLT_EN3MB= 2; // Experimental Ethernet (3Mb) 
          DLT_AX25= 3; // Amateur Radio AX.25
          DLT_PRONET= 4; // Proteon ProNET Token Ring 
          DLT_CHAOS= 5; // Chaos 
          DLT_IEEE802= 6; // IEEE 802 Networks 
          DLT_ARCNET= 7; // ARCNET 
          DLT_SLIP= 8; // Serial Line IP
          DLT_PPP= 9; // Point-to-point Protocol 
          DLT_FDDI= 10; // FDDI 
          DLT_ATM_RFC1483= 11; // LLC/SNAP encapsulated atm 
          DLT_RAW= 12; // raw IP 
          DLT_SLIP_BSDOS= 13; // BSD/OS Serial Line IP
          DLT_PPP_BSDOS= 14; // BSD/OS Point-to-point Protocol
          // New types for Win32
          DLT_EN100MB= 100; // Ethernet (100Mb)
          DLT_PPP_WIN32= 101; // Win32 dial up connection 

          // -- The default snapshot length.
          // -- This value allows most printers to
          // -- print useful information while keeping
          // -- the amount of unwanted data down.In
          // -- particular, it allows for an ethernet
          // -- header, tcp/ip header, and 14 bytes of
          // -- data (assuming no ip options).
          k_default_snapshot_length= 68;

    type // -- stats for diagnostics
         PPcap_Stat= ^TPcap_stat;
         Tpcap_stat= record
                       ps_recv, // number of packets received 
                       ps_drop, // number of packets dropped 
                       ps_ifdrop: LongWord; // drops by interface not supported 
                     end;

         // -- diagnostics
         TPcap_md= record
                     Stat: TPcap_stat;
                     use_bpf: integer;
                     TotPkts: LongWord; // Can owerflow after 79hours on ethernet
                     TotAccepted: LongWord; // accepted by filter/sniffer
                     TotDrops: LongWord; // dropped packets
                     TotMissed: Longword; // missed by i/f during this run
                     OrigMissed: LongWord; // missed by i/f before this run
                   end;

         // -- Save file
         TPcap_sf= record
                     rfile: HFILE;
                     swapped: integer;
                     version_major: integer;
                     Version_Minor: integer;
                     base: Pointer;
                   end;

         // -- packetHeader
         PPcap_PktHdr= ^Tpcap_pkthdr;
         TPcap_pkthdr= record
                         ts: TUnixTimeVal; // Time of capture
                         CapLen, // captured length
                         Len: Integer; // actual length of packet
                       end;

         t_pt_capture_info= ^t_capture_info;
         t_capture_info= record
                           m_pt_adapter: Padapter;
                           m_pt_packet: PPacket; // Global Driver packet. kind of a buffer
                           m_shapshot_length: integer;
                           m_link_type: integer; // Type and speed of net
                           m_time_zone_offset: integer; // timezone offset
                           offset: integer;
                           sf: Tpcap_sf; // Save file
                           md: Tpcap_md; // Diagnostics
                           // -- read buffer
                           m_buffer_size: integer;
                           m_pt_buffer: Pointer; //u_char
                           m_start_pointer: Pointer; //u_char
                           m_remaining_byte_count: integer;
                           // -- dummy pcap_next().
                           pkt: Pointer; //U_char
                           // -- dummy Placeholder for filter code if bpf not in kernel.
                           fcode: array[0..1] of Integer; // Tbpf_program;
                           errbuf: array[0..PCAP_ERRBUF_SIZE- 1] of char; //Last error message
                         end;

         // -- callback procedure
         Tpcap_handler= procedure (User: pointer; const Header: Ppcap_pkthdr; const Data: pchar);

    function Pcap_getAdapternames(Delimiter: char; var ErrStr: string): string;

    procedure pcap_set_read_timeout(p_pt_pcap_info: t_pt_capture_info; p_timeout: Integer);
    function pcap_open_live(p_adapter_name: String; p_shapshot_length: LongWord;
        p_ndis_promiscuous_mode: boolean;
        p_timeout_ms: integer; var errstr: String): t_pt_capture_info;
    function pcap_read(p_pt_pcap_info: t_pt_capture_info; p_packet_count_max: integer; CallBack: Tpcap_handler; User:
          pointer): integer;
    procedure pcap_close(var pv_pt_pcap_info: t_pt_capture_info);

  implementation
    uses u_c_display, u_display_hex_2;

    function Pcap_GetAdapternames(Delimiter: char; var ErrStr: string): string;
        // -- Get All AdapterNames seperated with chosen delimiter // Added By Lars Peter
      type // -- Eases the process of determing the
           // -- platform and do proper instructions
           // -- I.e : Char vs. WideChar issue
           t_windows_version= (wv_WinS, wv_Win9x,
               wv_WinNT, wv_Win2000, wv_WinXP,
               wv_Unknown);

      function f_get_windows_version(var VerStr: string): t_windows_version;
        var OSversion: OSVERSIONINFO;
        begin
          OSversion.dwOSVersionInfoSize:= sizeof(OSVERSIONINFO);
          Result:= wv_unknown;
          if not GetVersionEx(osversion)
            then exit;

          with OSversion do
          begin
            Case dwPlatformId of
              VER_PLATFORM_WIN32s: begin
                    verStr:= Format('Windows %d.%d', [dwMajorVersion,
                        dwMinorVersion]);
                    Result:= Wv_wins;
                  end;
              VER_PLATFORM_WIN32_WINDOWS: begin
                    case dwMinorVersion of
                      0: verstr:= 'Windows 95';
                      10: verstr:= 'Windows 98';
                      90: verstr:= 'Windows Me';
                    end;
                    Result:= Wv_win9x;
                  end;
              VER_PLATFORM_WIN32_NT: begin
                    if (dwMajorVersion= 5) and (dwMinorVersion= 0)
                      then
                        begin
                          verstr:= 'Windows 2000';
                          if szCSDVersion<> ''
                            then Verstr:= verstr+ ' with '+ szCSDVersion;
                          Result:= wv_win2000;
                        end
                      else
                        if (dwMajorVersion= 5) and (dwMinorVersion= 1)
                          then
                            begin
                              verstr:= Format('Windows XP %s', [szCSDVersion]);
                              if szCSDVersion<> ''
                                then Verstr:= verstr+ ' with '+ szCSDVersion;
                              Result:= wv_winxp;
                            end
                          else
                            if (dwMajorVersion<= 4)
                              then
                                begin
                                  verstr:= Format('Windows NT %d.%d', [
                                      dwMajorVersion, dwMinorVersion]);
                                  if szCSDVersion<> ''
                                    then Verstr:= verstr+ ' with '+ szCSDVersion;
                                  Result:= wv_winNT;
                                end
                              else verstr:= format('Windows %d.%d ', [
                                  //for newest windows version
                                  dwMajorVersion, dwMinorVersion]);
                  end;
            end; // Case dwPlatformId
          end; // OSversion
        end; // f_get_windows_version

      var NameList: Array[0..(1024* 2)- 1] of char;
          NameLength, i: Longword;
          l_windows_version: t_windows_version;
          l_version_name: string; // dummy;

      begin // Pcap_GetAdapternames
        NameLength:= 1024;

        PacketGetAdapterNames(NameList, @ NameLength);

        l_windows_version:= f_get_windows_version(l_version_name);
        // -- WINDOWS 95,98 or ME  8bits per character
        if l_windows_version= wv_Win9x
          then
            begin
              for i:= 0 to NameLength- 1 do
              begin
                // -- modified
                if ((NameList[i]= #0) and (NameList[i+ 1]= #0))
                  then break
                  else
                    if (NameList[i]= ' ') or (NameList[i]= #0)
                      then NameList[i]:= delimiter;
              end;
              Result:= NameList;
            end
          Else begin
              // -- WINDOWS NT,2000 or XP     16bits per character
              // display('xp');

              for i:= 0 to NameLength- 1 do
              begin
                if (Pwidechar(@NameList)[i]= #0) and (PwideChar(@namelist)[i+ 1]<> #0)
                  then PwideChar(@NameList)[i]:= WideChar(delimiter);
              end;

              Result:= WideCharToString(PWideChar(@NameList));
            end;
      end; // Pcap_GetAdapternames

    procedure pcap_set_read_timeout(p_pt_pcap_info: t_pt_capture_info; p_timeout: Integer);
      begin
        PacketSetReadTimeout(p_pt_pcap_info.m_pt_adapter, 100);
      end; // pcap_set_read_timeout

    function pcap_open_live(p_adapter_name: String; p_shapshot_length: LongWord;
        p_ndis_promiscuous_mode: boolean;
        p_timeout_ms: integer; var errstr: String): t_pt_capture_info;
      const k_capture_buffer_size= 256000;

      var l_pt_pcap_info: t_pt_capture_info;
          l_net_type: Tnet_type;
          l_adapter_name_z: Pchar;

      procedure cleanup;
        begin
          if l_pt_pcap_info.m_pt_adapter<> nil
            then PacketCloseAdapter(l_pt_pcap_info.m_pt_adapter);
          if l_pt_pcap_info.m_pt_buffer<> nil
            then FreeMem(l_pt_pcap_info.m_pt_buffer, k_capture_buffer_size);
          Freemem(l_pt_pcap_info, SizeOf(t_capture_info));
        end; // cleanup

      begin // pcap_open_live
        display('open '+ p_adapter_name);

        Result:= nil;

        // -- create the pcap record
        GetMem(l_pt_pcap_info, SizeOf(t_capture_info));
        if l_pt_pcap_info= nil
          then
            begin
              ErrStr:= 'Cannot allocate pcap object';
              exit;
            end;

        FillChar(l_pt_pcap_info^, sizeof(t_capture_info), 0);

        // -- create an adapter record
        GetMem(l_adapter_name_z, 2048); // Making temporary pchar
        StrPCopy(l_adapter_name_z, p_adapter_name);
        l_pt_pcap_info.m_pt_adapter:= PacketOpenAdapter(l_adapter_name_z);
        FreeMem(l_adapter_name_z, 2048);

        if l_pt_pcap_info.m_pt_adapter= nil
          then
            begin
              ErrStr:= 'cannot open adapter "'+ p_adapter_name+ '"';
              cleanup;
              exit;
            end;

        // -- set filter mode
        if p_ndis_promiscuous_mode
          then
            begin
              if not PacketSetHWFilter(l_pt_pcap_info.m_pt_adapter, NDIS_PACKET_TYPE_PROMISCUOUS)
                then
                  Begin
                    ErrStr:= 'cannot set adapter filter to promiscuous mode';
                    cleanup;
                    exit;
                  end;
            end
          else
            if not PacketSetHWFilter(l_pt_pcap_info.m_pt_adapter, NDIS_PACKET_TYPE_DIRECTED)
              then
                begin
                  ErrStr:= 'cannot set adapter filter to directed mode';
                  cleanup;
                  exit;
                end;

        // -- get card speed and type
        if not PacketGetNetType(l_pt_pcap_info.m_pt_adapter, @l_net_type)
          then
            Begin
              ErrStr:= 'cannot determine network type and speed';
              cleanup;
              exit;
            end;

        Case TNDIS_MEDIUM(l_net_type.linktype) of
          NdisMediumWan: l_pt_pcap_info.m_link_type:= DLT_PPP_WIN32;
          NdisMedium802_3: begin
                             if l_net_type.LinkSpeed= 100000000
                               then l_pt_pcap_info.m_link_type:= DLT_EN100MB
                               else
                                 if l_net_type.LinkSpeed= 10000000
                                   then l_pt_pcap_info.m_link_type:= DLT_EN10MB
                                   else l_pt_pcap_info.m_link_type:= DLT_PPP_WIN32;
                           end;
          else l_pt_pcap_info.m_link_type:= DLT_EN10MB;
        end;

        // -- allocate link header
        l_pt_pcap_info.m_buffer_size:= k_capture_buffer_size;
        GetMem(l_pt_pcap_info.m_pt_buffer, k_capture_buffer_size);
        if l_pt_pcap_info.m_pt_buffer= nil
          then
            begin
              ErrStr:= 'Cannot allocate Link Header space';
              cleanup;
              exit;
            end;
        l_pt_pcap_info.m_shapshot_length:= p_shapshot_length;

        // -- allocate global capture packet
        l_pt_pcap_info.m_pt_packet:= PacketAllocatePacket;
        if l_pt_pcap_info.m_pt_packet= nil
          then
            begin
              ErrStr:= 'Cannot allocate Global Packet Object';
              cleanup;
              exit;
            end;

        PacketInitPacket(l_pt_pcap_info.m_pt_packet, l_pt_pcap_info.m_pt_buffer, l_pt_pcap_info.m_buffer_size);

        // -- allocate driver buffer
        if not PacketSetBuff(l_pt_pcap_info.m_pt_adapter, DEFAULT_DRIVERBUFFER)
          then
            begin
              ErrStr:= 'Not enough memory to allocate Driver m_pt_buffer';
              cleanup;
              exit;
            end;

        Result:= l_pt_pcap_info;
      end; // pcap_open_live

    function pcap_read(p_pt_pcap_info: t_pt_capture_info; p_packet_count_max: integer; CallBack: Tpcap_handler;
        User: pointer): integer;
        // -- returns the number of packets read
      var l_character_count: Longword; // Counter ?
          l_packet_count: integer;
          l_start_pointer, l_pt_end_pointer: pointer; //Begin and End Point ?
          l_header_length: Integer;
          l_byte_count: integer; //Length of captured
      begin
        l_character_count:= p_pt_pcap_info.m_remaining_byte_count;

        l_packet_count:= 0;

        // -- if all bytes have been handled, fetch new packets
        if p_pt_pcap_info.m_remaining_byte_count= 0
          then
            begin
              // -- capture the packets
              if Not PacketReceivePacket(p_pt_pcap_info.m_pt_adapter, p_pt_pcap_info.m_pt_packet, True)
                then
                  begin
                    // ERROR!
                    p_pt_pcap_info.errbuf:= 'read error: PacketRecievePacket failed';
                    Result:= - 1;
                    exit;
                  end;

              l_character_count:= p_pt_pcap_info.m_pt_packet.ulBytesReceived;
              l_start_pointer:= p_pt_pcap_info.m_pt_buffer;
            end
          else l_start_pointer:= p_pt_pcap_info.m_start_pointer;

        // -- loop through each packet
        // --  initialize the end pointer
        l_pt_end_pointer:= ptr(longword(l_start_pointer)+ l_character_count);

        while (longword(l_start_pointer)< longword(l_pt_end_pointer)) do
        begin
          l_byte_count:= Pbpf_hdr(l_start_pointer).bh_caplen;
          l_header_length:= Pbpf_hdr(l_start_pointer).bh_hdrlen;

          // -- XXX A bpf_hdr matches apcap_pkthdr.
          callback(user, Ppcap_pkthdr(l_start_pointer), ptr(longword(l_start_pointer)+ l_header_length));

          LongWord(l_start_pointer):= LongWord(l_start_pointer)+ BPF_WORDALIGN(l_byte_count+ l_header_length);
          inc(l_packet_count);
          if (l_packet_count>= p_packet_count_max) and (p_packet_count_max> 0)
            then
              begin
                p_pt_pcap_info.m_start_pointer:= l_start_pointer;
                p_pt_pcap_info.m_remaining_byte_count:= longword(l_pt_end_pointer)- longword(l_start_pointer);
                Result:= l_packet_count;
                exit;
              end;
        end; // while bytes

        p_pt_pcap_info.m_remaining_byte_count:= 0;
        Result:= l_packet_count;
      end; // pcap_read

    procedure pcap_close(var pv_pt_pcap_info: t_pt_capture_info);
        // -- very simplified from the original
      begin
        if pv_pt_pcap_info= nil
          then exit;

        if pv_pt_pcap_info.m_pt_adapter<> nil
          then
            begin
              PacketCloseAdapter(pv_pt_pcap_info.m_pt_adapter);
              pv_pt_pcap_info.m_pt_adapter:= nil;
            end;

        if pv_pt_pcap_info.m_pt_buffer<> nil
          then
            begin
              FreeMem(pv_pt_pcap_info.m_pt_buffer, pv_pt_pcap_info.m_buffer_size);
              pv_pt_pcap_info.m_pt_buffer:= nil;
            end;
        FreeMem(pv_pt_pcap_info, sizeof(t_capture_info));
        pv_pt_pcap_info:= nil;
      end; // pcap_close

    end.
