// 003 u_ip_icmp_packet
// 10 dec 2004

// -- (C) Felix John COLIBRI 2004
// -- documentation: http://www.felix-colibri.com

(*$r+*)

unit u_ip_icmp_packet;
  interface
    uses Windows, u_sniffer_utils
      ;

    type t_ip_icmp_packet= packed record
                                 icmp_type: byte;
                                 icmp_code: byte;
                                 icmp_cksum: Word;
                                 filler: dWord;
                                 m_icmp_data: Byte;
                               end;
         t_pt_ip_icmp_packet= ^t_ip_icmp_packet;

    const // -- 8 (ethernet+ ip+ icmp= 42)
          k_ip_icmp_header_size= SizeOf(t_ip_icmp_packet)- 1;

         // -- ICMP message types  as defined in
         // --   RFC792,RFC950,RFC1256,RFC1393,RFC1475
         ICMP_ECHOREPLY           = 0  ;  // echo reply
         ICMP_UNASSIGNED_1        = 1  ;  // unassigned
         ICMP_UNASSIGNED_2        = 2  ;  // unassigned
         ICMP_UNREACH             = 3  ;  // dest unreachable
         ICMP_SOURCEQUENCH        = 4  ;  // packet lost, slow down
         ICMP_REDIRECT            = 5  ;  // shorter route
         ICMP_ALT_HOST_ADDR       = 6  ;  // alternate host address
         ICMP_UNASSIGNED_7        = 7  ;  // unassigned
         ICMP_ECHO                = 8  ;  // echo service
         ICMP_RTR_ADVERTISEMENT   = 9  ;  // router advertisement
         ICMP_RTR_SOLLICITATION   = 10 ;  // router sollicitation;
         ICMP_TIMXCEED            = 11 ;  // time exceeded
         ICMP_PARAMPROB           = 12 ;  // ip packet bad
         ICMP_TSTAMP              = 13 ;  // timestamp request
         ICMP_TSTAMPREPLY         = 14 ;  // timestamp reply
         ICMP_IREQ                = 15 ;  // information request
         ICMP_IREQREPLY           = 16 ;  // information reply
         ICMP_MASKREQ             = 17 ;  // address mask request
         ICMP_MASKREPLY           = 18 ;  // address mask reply

         //  ICMP message names
         k_ip_icmp_message_names: array[0..18] of string =
            ( 'echo_reply', 'UNASSIGNED', 'UNASSIGNED',
              'dest_unreachable', 'source_quench',
              'redirect',
              // -- 6
              'alternate_host_address', 'UNASSIGNED', 'echo_request',
              'router_advertisement', 'router_sollicitation',
              // -- 11
              'time_exceeded',
              'param_problem', 'timestamp_request','timestamp_reply',
              'info_request',
              // -- 16
              'info_reply', 'addr.mask_request', 'addr.mask_reply'
            );

(*

    type c_display_ip_icmp_packet= class
                                     m_icmp_size: Integer;
                                     m_pt_ip_icmp_packet: t_pt_ip_icmp_packet;

                                     Constructor create_display_ip_icmp_packet;
                                     function f_c_packet(p_pt_ip_icmp_packet: t_pt_ip_icmp_packet;
                                         p_icmp_size: Integer): c_display_ip_icmp_packet;
                                     function f_size: String;
                                     function f_icmp_message_name: String;

                                     function f_display_destination_unreachable: String;
                                     function f_display_icmp_data: String;
                                 end;
*)
  implementation
    uses SysUtils, u_types_constants, u_c_display, u_display_hex_2;

(*
    // -- c_ip_icmp_packet

    Constructor c_display_ip_icmp_packet.create_display_ip_icmp_packet;
      begin
        // create_ip_sub_packet(p_name, p_icmp_size, t_pt_bytes(p_pt_ip_icmp_packet));

      end; // create_display_ip_icmp_packet

    function c_display_ip_icmp_packet.f_c_packet(p_pt_ip_icmp_packet: t_pt_ip_icmp_packet;
        p_icmp_size: Integer): c_display_ip_icmp_packet;
      begin
        m_icmp_size:= p_icmp_size;
        m_pt_ip_icmp_packet:= p_pt_ip_icmp_packet;

        Result:= Self;
      end; // f_c_packet

    function c_display_ip_icmp_packet.f_size: String;
      begin
        Result:= IntToStr(m_icmp_size);
      end; // f_size

    function c_display_ip_icmp_packet.f_icmp_message_name: String;
      begin
        Result:= k_ip_icmp_message_names[m_pt_ip_icmp_packet^.icmp_type];
      end; // f_icmp_message_name

    function c_display_ip_icmp_packet.f_display_destination_unreachable: String;
      begin
        with m_pt_ip_icmp_packet^ do
        begin
          Result:= IntToStr(icmp_code);
          case icmp_code of
            0 : Result:= 'Netword_unreachable';

          end; // case

          Result:= Result+ f_display_hex(@ m_icmp_data, m_icmp_size- k_ip_icmp_header_size, 16, k_hex_ascii);
        end; // with m_pt_ip_icmp_packet
      end; // f_display_destination_unreachable

    function c_display_ip_icmp_packet.f_display_icmp_data: String;
      begin
        with m_pt_ip_icmp_packet^ do
        begin
          Result:= IntToStr(icmp_code)+ k_new_line;
          Result:= Result+ f_display_hex(@ m_icmp_data, m_icmp_size- k_ip_icmp_header_size, 16, k_hex_ascii);
        end; // with m_pt_ip_icmp_packet
      end; // c_display_ip_icmp_packet.
*)
    begin // u_c_ip_icmp_packet
    end. // u_c_ip_icmp_packet
cket
