// 003 u_ip_tcp_packet
// 10 dec 2004

// -- (C) Felix John COLIBRI 2004
// -- documentation: http://www.felix-colibri.com

(*$r+*)

unit u_ip_tcp_packet;
  interface
    uses Windows, u_sniffer_utils
      // , u_c_session_list

      ;

    // -- IP - TCP protocol

    // -- default MPU packet of ? 576 (14+ 20+ 20+ 522)
    // -- t_ip_tcp_packet = "tcp message" or "tcp segment"

    type t_ip_tcp_packet= packed record
                                   m_tcp_source_port: Word; // endian
                                   m_tcp_destination_port: Word; // endian

                                   m_tcp_sequence: dWord;
                                   m_tcp_acknowledgement: dWord;

                                   m_tcp_reserved_and_header_size: Byte;
                                   m_tcp_flags_and_reserved_2: Byte;
                                   m_tcp_window_size: Word;
                                   m_tcp_checksum: Word;
                                   m_tcp_last_urgent_byte: Word;

                                   // -- variable size
                                   m_tcp_data: array[0..0] of Byte;
                                 end;
         t_pt_ip_tcp_packet= ^t_ip_tcp_packet;

    const k_ip_tcp_header_size= SizeOf(t_ip_tcp_packet)- 1;

    const TCP_FLAG_FIN= $01; // the sender has finished
          TCP_FLAG_SYN= $02; // synchronize the sequence numbers
          TCP_FLAG_RST= $04; // reset the tcp connecton
          TCP_FLAG_PSH= $08; // send data immediately
          TCP_FLAG_ACK= $10; // m_tcp_acknowlegement is correct
          TCP_FLAG_URG= $20; // urgent

          k_ip_tcp_fag_names : array[1..6] of string =
              ('fin', 'syn', 'rst', 'psh', 'ack', 'urg');

    function f_flags(p_tcp_flags: Byte): string;
    function f_acknowledge_and_sequence(p_pt_ip_tcp_packet: t_pt_ip_tcp_packet): String;
    function f_flags_acknowledge_and_sequence(p_pt_ip_tcp_packet: t_pt_ip_tcp_packet): String;

(*
    function f_next_sequence(p_id: Integer; p_direction: Char;
        p_pt_ip_tcp_packet: t_pt_ip_tcp_packet; p_tcp_size: Integer;
        p_c_session: c_session): String;
*)
  implementation
    uses SysUtils, u_c_display, u_display_hex_2, u_types_constants;

    function f_flags(p_tcp_flags: Byte): string;
      var l_flag: integer;
      begin
        Result := '';

        for l_flag:= 1 to 6 do
        begin
          if p_tcp_flags and 1 =1
            then Result := Result + k_ip_tcp_fag_names[l_flag]+ ',';
          p_tcp_flags:= p_tcp_flags shr 1;
        end; // for l_flag
      end; // f_flags

    function f_acknowledge_and_sequence(p_pt_ip_tcp_packet: t_pt_ip_tcp_packet): String;
      begin
        with p_pt_ip_tcp_packet^ do
        begin
          if m_tcp_flags_and_reserved_2 and $10> 0
            then Result:= 'ack= '+ IntToHex(m_tcp_acknowledgement, 8)
            else Result:= '';
          Result:= Result+ ', seq= '+ IntToHex(m_tcp_sequence, 8);
        end; // with m_pt_ip_tcp_packet^
      end; // f_acknowledge_and_sequence

    function f_flags_acknowledge_and_sequence(p_pt_ip_tcp_packet: t_pt_ip_tcp_packet): String;
      var l_flag: integer;
          l_tcp_flags: Byte;
          l_tcp_acknowledgement, l_tcp_sequence: Integer;
      begin
        with p_pt_ip_tcp_packet^ do
        begin
          Result := '';

          l_tcp_flags:= m_tcp_flags_and_reserved_2;
          for l_flag:= 1 to 6 do
          begin
            if l_tcp_flags and 1 =1
              then Result := Result + k_ip_tcp_fag_names[l_flag]+ ' '
              else Result:= Result+ '    ';
            l_tcp_flags:= l_tcp_flags shr 1;
          end; // for l_flag

          l_tcp_acknowledgement:= f_swap_dword(pChar(@ m_tcp_acknowledgement));
          l_tcp_sequence:= f_swap_dword(pChar(@ m_tcp_sequence));

          if m_tcp_flags_and_reserved_2 and $10> 0
            then Result:= Result+ Format('%8x', [l_tcp_acknowledgement])
            else Result:= Result+ '        ';

          Result:= Result+ ' '+ Format('%8x', [l_tcp_sequence]);
        end; // with p_pt_ip_tcp_packet
      end; // f_flags_acknowledge_and_sequence
{
    function f_next_sequence(p_id: Integer; p_direction: Char;
        p_pt_ip_tcp_packet: t_pt_ip_tcp_packet; p_tcp_size: Integer;
        p_c_session: c_session): String;
      var l_tcp_acknowledgement, l_tcp_sequence: Integer;
      begin
        if p_tcp_size= 0
          then begin
              Result:= ' empty';
              Exit;
            end;

        with p_pt_ip_tcp_packet^ do
        begin
          l_tcp_acknowledgement:= f_swap_dword(pChar(@ m_tcp_acknowledgement));
          l_tcp_sequence:= f_swap_dword(pChar(@ m_tcp_sequence));

          if p_direction= 'i'
            then begin
                p_c_session.add_the_packet(p_id, p_direction, l_tcp_sequence, t_pt_bytes(@ p_pt_ip_tcp_packet^.m_tcp_data), p_tcp_size);

                // Inc(l_tcp_sequence, p_tcp_size);
                // Result:= Result+ Format('  %8x', [l_tcp_sequence]);

                Result:= Result+ ' i '+ p_c_session.m_display_reception;
              end
            else begin
                if p_c_session= Nil
                  then display_bug_halt('p_c_session Nil');

                p_c_session.add_the_packet(p_id, p_direction, l_tcp_sequence, t_pt_bytes(@ p_pt_ip_tcp_packet^.m_tcp_data), p_tcp_size);

                // Inc(l_tcp_sequence, p_tcp_size);
                // Result:= Result+ Format('           %8x', [l_tcp_sequence]);
                Result:= Result+ ' o '+ p_c_session.m_display_emission;
              end;
        end; // with p_pt_ip_tcp_packet
      end; // f_next_sequence
}
    begin // u_c_ip_tcp_packet
    end. // u_c_ip_tcp_packet
cket
